@@ -23,7 +23,7 @@ struct gengetopt_option
   char * var_arg; /* canonized long_opt + "_arg" = argument var */
   int default_given ; /* if a default is given */
   char * default_string ; /* default value for this option, if string */
-  char * group_value;
+  char * group_value; /* group name, if it's part of an option group */
   int multiple; /* whether this option can be given more than once */
 };
 
