@@ -25,6 +25,10 @@
 
 #include <fstream>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 extern "C"
 {
 #include "argsdef.h"
@@ -39,6 +43,7 @@ extern "C"
 #include "string_opt_arg.h"
 #include "int_opt_arg.h"
 #include "long_opt_arg.h"
+#include "long_long_opt_arg.h"
 #include "float_opt_arg.h"
 #include "flag_opt_arg.h"
 #include "required_option.h"
@@ -55,9 +60,14 @@ extern "C"
 #include "multiple_opt_list.h"
 #include "multiple_option.h"
 #include "multiple_fill_array.h"
+#include "gen_strdup.h"
 
 #include "conf_parser_gen.h"
 
+#ifndef HAVE_STRDUP
+extern "C" char *strdup (const char *s) ;
+#endif
+
 using std::ofstream;
 using std::endl;
 using std::string;
@@ -65,16 +75,19 @@ using std::string;
 extern char * gengetopt_package;
 extern char * gengetopt_version;
 extern char * gengetopt_purpose;
-extern int gengetopt_strdup_text_length;
-extern char *gengetopt_strdup_text[];
+extern char * gengetopt_input_filename;
 
 extern groups_collection_t gengetopt_groups;
 
 static char *create_filename (char *name, char *ext);
 static ofstream *open_fstream (char *filename);
 static char *canonize_names(const char * name);
+static const string canonize_name(const string &name);
 static const string strip_path(const string &);
 static const string to_upper(const string &);
+static void _generate_option_arg(ostream &stream, 
+                                 unsigned int indent,
+                                 struct gengetopt_option * opt);
 
 char *
 create_filename (char *name, char *ext)
@@ -119,7 +132,8 @@ CmdlineParserCreator::CmdlineParserCreator (char *function_name,
                                             bool no_handle_error_,
                                             bool conf_parser_,
                                             const string &comment_) :
-  filename (filename_), comment (comment_),
+  filename (filename_), 
+  comment (comment_),
   unamed_options (unamed_options_), 
   long_help (long_help_), no_handle_help (no_handle_help_),
   no_handle_version (no_handle_version_), no_handle_error (no_handle_error_),
@@ -131,16 +145,34 @@ CmdlineParserCreator::CmdlineParserCreator (char *function_name,
   header_filename = create_filename (filename, header_ext);
 
   // header_gen_class
-  const string header_file_name = strip_path (filename);
-  set_header_file_name (header_file_name);
+  const string stripped_header_file_name = strip_path (filename);
+  set_header_file_name (stripped_header_file_name);
   header_gen_class::set_generator_version (VERSION);
-  const string ifndefname = to_upper (strip_path (header_file_name));
-  set_ifndefname (canonize_names (ifndefname.c_str ()));
+  const string my_ifndefname = 
+    to_upper (strip_path (stripped_header_file_name));
+  set_ifndefname (canonize_names (my_ifndefname.c_str ()));
   header_gen_class::set_parser_name (parser_function_name);
+  const string my_package_var_name = 
+    to_upper (parser_function_name) + "_PACKAGE";
+  const string my_version_var_name = 
+    to_upper (parser_function_name) + "_VERSION";
+  header_gen_class::set_package_var_name (my_package_var_name);
+  c_source_gen_class::set_package_var_name (my_package_var_name);
+  header_gen_class::set_version_var_name (my_version_var_name);
+  c_source_gen_class::set_version_var_name (my_version_var_name);
+  const string uppersand = "\"";
+
   if (gengetopt_package)
-    set_if_package_notdefined (gengetopt_package);
+    set_package_var_val
+      (uppersand + gengetopt_package + uppersand);
+  else
+    set_package_var_val ("PACKAGE");
+
   if (gengetopt_version)
-    set_if_version_notdefined (gengetopt_version);
+    set_version_var_val
+      (uppersand + gengetopt_version + uppersand);
+  else
+    set_version_var_val ("VERSION");
 
   if (conf_parser)
     {
@@ -185,6 +217,8 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
   int_gen.set_optarg (argstr);
   long_opt_arg_gen_class long_gen;
   long_gen.set_optarg (argstr);
+  long_long_opt_arg_gen_class long_long_gen;
+  long_long_gen.set_optarg (argstr);
   float_opt_arg_gen_class float_gen;
   float_gen.set_optarg (argstr);
   flag_opt_arg_gen_class flag_gen;
@@ -194,6 +228,7 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
       int_gen.set_structure (string (opt->var_arg) + "_new");
       str_gen.set_structure (string (opt->var_arg) + "_new");
       long_gen.set_structure (string (opt->var_arg) + "_new");
+      long_long_gen.set_structure (string (opt->var_arg) + "_new");
       float_gen.set_structure (string (opt->var_arg) + "_new");
     }
   else
@@ -201,6 +236,7 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
       int_gen.set_structure (ARGS_STRUCT);
       str_gen.set_structure (ARGS_STRUCT);
       long_gen.set_structure (ARGS_STRUCT);
+      long_long_gen.set_structure (ARGS_STRUCT);
       float_gen.set_structure (ARGS_STRUCT);
     }
 
@@ -256,9 +292,8 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
       generateBreak(stream, indent);
       break;
     case ARG_LONGLONG:
-      long_gen.set_cast ("(long long)");
-      long_gen.set_opt_var (opt->var_arg);
-      long_gen.generate_long_opt_arg (stream, indent);
+      long_long_gen.set_opt_var (opt->var_arg);
+      long_long_gen.generate_long_long_opt_arg (stream, indent);
       generateBreak(stream, indent);
       break;
     default:
@@ -326,41 +361,70 @@ CmdlineParserCreator::generate_option_arg(ostream &stream,
         else
           first = false;
 
-        stream << arg_types[opt->type];
-	if (opt->multiple)
-	  stream << "* ";
-	else
-	  stream << " ";
+        _generate_option_arg (stream, indent, opt);
+
         break;
-      default: fprintf (stderr, "gengetopt: bug found in %s:%d!!\n",
-                        __FILE__, __LINE__);
+      default: 
+        fprintf (stderr, "gengetopt: bug found in %s:%d!!\n",
+                 __FILE__, __LINE__);
         abort ();
       }
-      
-      if (opt->type == ARG_FLAG)
-        stream << opt->var_arg << "_flag";
-      else
-        stream << opt->var_arg << "_arg";
-      
-      stream << ";\t/* " << opt->desc;
-      
-      if (opt->default_given)
-        {
-          stream << " (default='" << opt->default_string << "')";
-        }
-      
-      if (opt->type == ARG_FLAG)
-        {
-          if (opt->flagstat)
-            stream << " (default=on)";
-          else
-            stream << " (default=off)";
-        }
-      
-      stream << ".  */\n";
     }
 }
 
+void 
+_generate_option_arg(ostream &stream, 
+                     unsigned int indent,
+                     struct gengetopt_option *opt)
+{
+  ostringstream opt_stream;
+
+  if (opt->multiple)
+    opt_stream << "* ";
+  else
+    opt_stream << " ";
+      
+  if (opt->type == ARG_FLAG)
+    opt_stream << opt->var_arg << "_flag";
+  else
+    opt_stream << opt->var_arg << "_arg";
+  
+  opt_stream << ";\t/* " << opt->desc;
+      
+  if (opt->default_given)
+    {
+      opt_stream << " (default='" << opt->default_string << "')";
+    }
+      
+  if (opt->type == ARG_FLAG)
+    {
+      if (opt->flagstat)
+        opt_stream << " (default=on)";
+      else
+        opt_stream << " (default=off)";
+    }
+      
+  opt_stream << ".  */\n";
+  const string opt_string = opt_stream.str ();
+
+  if (opt->type == ARG_LONGLONG)
+    {
+      string indent_str (indent, ' ');
+
+      stream << "#ifdef HAVE_LONG_LONG\n";
+      stream << indent_str;
+      stream << arg_types[opt->type] << opt_string;
+      stream << indent_str;
+      stream << "#else\n";
+      stream << indent_str;
+      stream << arg_types[ARG_LONG] << opt_string;
+      stream << indent_str;
+      stream << "#endif\n";
+    }
+  else
+    stream << arg_types[opt->type] << opt_string;
+}
+
 void 
 CmdlineParserCreator::generate_option_given(ostream &stream, 
                                             unsigned int indent)
@@ -631,10 +695,10 @@ CmdlineParserCreator::generate_strdup(ostream &stream, unsigned int indent)
   stream << endl;
   if (gen_strdup) 
     {
-      stream << "#ifndef HAVE_STRDUP\n";
-      for (int i = 1; i <= gengetopt_strdup_text_length; ++i)
-        stream << gengetopt_strdup_text[i] << endl;
-      stream << "#endif /* HAVE_STRDUP */\n";
+      gen_strdup_gen_class strdup_gen;
+
+      strdup_gen.set_prefix ("gengetopt");
+      strdup_gen.generate_gen_strdup (stream);
     }
 }
 
@@ -718,7 +782,7 @@ CmdlineParserCreator::generate_clear_arg(ostream &stream, unsigned int indent)
 
           if (opt->default_given)
             stream << ARGS_STRUCT << "->" << opt->var_arg 
-                   << "_arg = strdup(\"" << opt->default_string 
+                   << "_arg = gengetopt_strdup(\"" << opt->default_string 
                    << "\") ;\\";
           else
             stream << ARGS_STRUCT << "->" << opt->var_arg 
@@ -794,14 +858,14 @@ string
 CmdlineParserCreator::generate_getopt_string()
 {
   struct gengetopt_option * opt;
-  ostringstream getopt_string;
+  ostringstream built_getopt_string;
 
   foropt
     if (opt->short_opt)
-      getopt_string << opt->short_opt <<
+      built_getopt_string << opt->short_opt <<
         (opt->type == ARG_NO || opt->type == ARG_FLAG ? "" : ":");
 
-  return getopt_string.str ();
+  return built_getopt_string.str ();
 }
 
 void
@@ -816,6 +880,7 @@ CmdlineParserCreator::generate_handle_help(ostream &stream,
      help_gen.set_option_comment (HELP_OPT_DESCR);
      help_gen.set_option_var_name (HELP_LONG_OPT);
      help_gen.set_gen_exit ("return");
+     help_gen.set_package_var_name (c_source_gen_class::package_var_name);
 
      help_gen.generate_generic_option (stream, indent);
      
@@ -842,6 +907,7 @@ CmdlineParserCreator::generate_handle_version(ostream &stream,
      version_gen.set_option_comment (VERSION_OPT_DESCR);
      version_gen.set_option_var_name (VERSION_LONG_OPT);
      version_gen.set_gen_exit ("return");
+     version_gen.set_package_var_name (c_source_gen_class::package_var_name);
 
      version_gen.generate_generic_option (stream, indent);
      
@@ -864,6 +930,7 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
   no_short_option_gen_class opt_gen;
   string indent_str (indent, ' ');
   opt_gen.set_gen_exit (no_handle_error ? "return" : "exit");
+  opt_gen.set_package_var_name (c_source_gen_class::package_var_name);
   bool first = true;
 
   foropt
@@ -884,7 +951,8 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
           {
             str_stream.str ("");
             str_stream << " ";
-            group_gen.set_group_var_name (opt->group_value);
+            group_gen.set_group_var_name 
+              (canonize_name (opt->group_value));
             group_gen.generate_generic_option_group (str_stream, indent);
             opt_gen.set_update_group_count (str_stream.str ());
           }
@@ -912,6 +980,8 @@ CmdlineParserCreator::generate_handle_option(ostream &stream,
   multiple_option_gen_class multi_gen;
   string indent_str (indent, ' ');
 
+  option_gen.set_package_var_name (c_source_gen_class::package_var_name);
+
   if (no_handle_error)
     option_gen.set_gen_exit ("return");
   else
@@ -947,7 +1017,8 @@ CmdlineParserCreator::generate_handle_option(ostream &stream,
           generic_option_group_gen_class group_gen;
           if (opt->group_value)
             {
-              group_gen.set_group_var_name (opt->group_value);
+              group_gen.set_group_var_name 
+                (canonize_name (opt->group_value));
               group_gen.generate_generic_option_group (stream, indent);
             }
 
@@ -970,13 +1041,15 @@ CmdlineParserCreator::generate_handle_group(ostream &stream,
 {
   group_option_gen_class opt_gen;
   string indent_str (indent, ' ');
+  opt_gen.set_package_var_name (c_source_gen_class::package_var_name);
 
   groups_collection_t::const_iterator end = gengetopt_groups.end();
   for ( groups_collection_t::const_iterator idx = gengetopt_groups.begin();
         idx != end; ++idx)
     {
       stream << indent_str;
-      opt_gen.set_group_var_name (idx->first);
+      opt_gen.set_group_name (idx->first);
+      opt_gen.set_group_var_name (canonize_name (idx->first));
       if (idx->second)
         {
           opt_gen.set_Comparison_rule(GROUP_REQUIRED_COMPARISON);
@@ -999,6 +1072,7 @@ CmdlineParserCreator::generate_handle_required(ostream &stream,
 {
   struct gengetopt_option * opt;
   required_option_gen_class opt_gen;
+  opt_gen.set_package_var_name (c_source_gen_class::package_var_name);
   string indent_str (indent, ' ');
 
   /* write test for required options */
@@ -1033,7 +1107,7 @@ CmdlineParserCreator::generate_group_counters(ostream &stream,
         idx != end; ++idx)
     {
       stream << indent_str;
-      counter_gen.set_group_name (idx->first);
+      counter_gen.set_group_name (canonize_name (idx->first));
       counter_gen.generate_group_counter (stream, indent);
       stream << endl;
     }
@@ -1081,6 +1155,7 @@ CmdlineParserCreator::generate_conf_parser(ostream &stream,
   ConfigParserGenerator conf_parser_gen;
   conf_parser_gen.set_parser_name (c_source_gen_class::parser_name);
   conf_parser_gen.set_gen_exit (gen_exit);
+  conf_parser_gen.set_package_var_name (c_source_gen_class::package_var_name);
   conf_parser_gen.generate_config_parser_source (stream);
 }
 
@@ -1130,3 +1205,15 @@ to_upper(const string &old)
   return upper;
 }
 
+const string
+canonize_name(const string &old)
+{
+  string canonized = old;
+
+  for (string::iterator s = canonized.begin ();
+       s != canonized.end (); ++s)
+    if (*s == '.' || *s == '-' || *s == ' ') *s = '_';
+
+  return canonized;
+}
+
