@@ -1,4 +1,4 @@
-/* A Bison parser, made from parser.yy
+/* A Bison parser, made from /home/bettini/work/gengetopt/src/parser.yy
    by GNU bison 1.35.  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -38,13 +38,13 @@ static int gengetopt_package_given = 0;
 static int gengetopt_version_given = 0;
 static int gengetopt_purpose_given = 0;
 
-extern void yyerror ( char *error ) ;
-extern void yyerror2 ( char *error ) ;
+extern void yyerror ( const char *error ) ;
+extern void yyerror2 ( const char *error ) ;
 extern int yylex () ;
 
 //#define YYERROR_VERBOSE 1
 #define check_result \
-	if (o) { switch (o) { case 1: yyerror2 ("not enough memory"); break; case 2: yyerror2 ("long option redefined"); break; case 3: yyerror2 ("short option redefined"); break; case 4: yyerror2 ("bug found!!"); break; case 5: yyerror2 ("group undefined"); break; case 6: yyerror2("invalid default value"); break;} YYERROR; }
+	if (o) { switch (o) { case 1: yyerror2 ("not enough memory"); break; case 2: yyerror2 ("long option redefined"); break; case 3: yyerror2 ("short option redefined"); break; case 4: yyerror2 ("bug found!!"); break; case 5: yyerror ("group undefined"); break; case 6: yyerror2("invalid default value"); break;} YYERROR; }
 
 #line 47 "parser.yy"
 #ifndef YYSTYPE
