@@ -1,5 +1,5 @@
-#ifndef BISON_PARSER_H
-# define BISON_PARSER_H
+#ifndef BISON_Y_TAB_H
+# define BISON_Y_TAB_H
 
 #ifndef YYSTYPE
 typedef union {
@@ -32,4 +32,4 @@ int boolean;
 
 extern YYSTYPE yylval;
 
-#endif /* not BISON_PARSER_H */
+#endif /* not BISON_Y_TAB_H */
