@@ -20,6 +20,9 @@ main (int argc, char **argv)
 
   printf ("Value of foo: %d\n", args_info.foo_arg);
   printf ("Value of bar: %s\n", args_info.bar_arg);
+  printf ("Value of foofloat: %f\n", args_info.foofloat_arg);
+  printf ("Value of foodouble: %lf\n", args_info.foodouble_arg);
+  printf ("Value of PI: %16.10f\n", args_info.pi_arg);
 
   return 0;
 }
